@@ -1,4 +1,4 @@
-/* Copyright 2012--2018 The Tor Project
+/* Copyright 2012--2020 The Tor Project
  * See LICENSE for licensing information */
 
 package org.torproject.descriptor.impl;
@@ -21,6 +21,8 @@ import java.util.TreeMap;
 public class TorperfResultImpl extends DescriptorImpl
     implements TorperfResult {
 
+  private static final long serialVersionUID = 8961567618137500044L;
+
   protected static List<Descriptor> parseTorperfResults(
       byte[] rawDescriptorBytes, File descriptorFile)
       throws DescriptorParseException {
@@ -30,28 +32,31 @@ public class TorperfResultImpl extends DescriptorImpl
     List<Descriptor> parsedDescriptors = new ArrayList<>();
     String descriptorString = new String(rawDescriptorBytes,
         StandardCharsets.UTF_8);
-    Scanner scanner = new Scanner(descriptorString).useDelimiter("\r?\n");
-    String typeAnnotation = "";
-    while (scanner.hasNext()) {
-      String line = scanner.next();
-      if (line.startsWith("@type torperf ")) {
-        String[] parts = line.split(" ");
-        if (parts.length != 3) {
-          throw new DescriptorParseException("Illegal line '" + line
-              + "'.");
+    try (Scanner scanner =
+                 new Scanner(descriptorString).useDelimiter("\r?\n")) {
+      String typeAnnotation = "";
+      while (scanner.hasNext()) {
+        String line = scanner.next();
+        if (line.startsWith("@type torperf ")) {
+          String[] parts = line.split(" ");
+          if (parts.length != 3) {
+            throw new DescriptorParseException("Illegal line '" + line
+                    + "'.");
+          }
+          String version = parts[2];
+          if (!version.startsWith("1.")) {
+            throw new DescriptorParseException("Unsupported version in "
+                    + " line '" + line + "'.");
+          }
+          typeAnnotation = line + "\n";
+        } else {
+          /* XXX21932 */
+          parsedDescriptors.add(new TorperfResultImpl(
+                  (typeAnnotation + line + "\n")
+                          .getBytes(StandardCharsets.UTF_8),
+                  descriptorFile));
+          typeAnnotation = "";
         }
-        String version = parts[2];
-        if (!version.startsWith("1.")) {
-          throw new DescriptorParseException("Unsupported version in "
-              + " line '" + line + "'.");
-        }
-        typeAnnotation = line + "\n";
-      } else {
-        /* XXX21932 */
-        parsedDescriptors.add(new TorperfResultImpl(
-            (typeAnnotation + line + "\n").getBytes(StandardCharsets.UTF_8),
-            descriptorFile));
-        typeAnnotation = "";
       }
     }
     return parsedDescriptors;
