@@ -1,4 +1,4 @@
-/* Copyright 2012--2018 The Tor Project
+/* Copyright 2012--2020 The Tor Project
  * See LICENSE for licensing information */
 
 package org.torproject.descriptor.impl;
@@ -338,6 +338,15 @@ public class ServerDescriptorImplTest {
       return db.buildDescriptor();
     }
 
+    private String bridgeDistributionRequestLine = null;
+
+    private static ServerDescriptor createWithBridgeDistributionRequestLine(
+        String line) throws DescriptorParseException {
+      DescriptorBuilder db = new DescriptorBuilder();
+      db.bridgeDistributionRequestLine = line;
+      return db.buildDescriptor();
+    }
+
     private byte[] buildDescriptorBytes() {
       StringBuilder sb = new StringBuilder();
       if (this.routerLine != null) {
@@ -385,6 +394,9 @@ public class ServerDescriptorImplTest {
       if (this.contactLine != null) {
         sb.append(this.contactLine).append("\n");
       }
+      if (this.bridgeDistributionRequestLine != null) {
+        sb.append(this.bridgeDistributionRequestLine).append("\n");
+      }
       if (this.familyLine != null) {
         sb.append(this.familyLine).append("\n");
       }
@@ -453,6 +465,9 @@ public class ServerDescriptorImplTest {
     private ServerDescriptorImpl buildDescriptor()
         throws DescriptorParseException {
       byte[] descriptorBytes = this.buildDescriptorBytes();
+      if (null == descriptorBytes) {
+        descriptorBytes = new byte[0];
+      }
       return new RelayServerDescriptorImpl(descriptorBytes,
           new int[] { 0, descriptorBytes.length }, null);
     }
@@ -464,28 +479,28 @@ public class ServerDescriptorImplTest {
     ServerDescriptor descriptor = db.buildDescriptor();
     assertEquals("saberrider2008", descriptor.getNickname());
     assertEquals("94.134.192.243", descriptor.getAddress());
-    assertEquals(9001, (int) descriptor.getOrPort());
-    assertEquals(0, (int) descriptor.getSocksPort());
-    assertEquals(0, (int) descriptor.getDirPort());
+    assertEquals(9001, descriptor.getOrPort());
+    assertEquals(0, descriptor.getSocksPort());
+    assertEquals(0, descriptor.getDirPort());
     assertEquals("Tor 0.2.2.35 (git-b04388f9e7546a9f) on Linux i686",
         descriptor.getPlatform());
-    assertEquals(new TreeSet<Long>(Arrays.asList(
-        new Long[] { 1L, 2L, 3L, 4L })), descriptor.getProtocols().get("Link"));
-    assertEquals(new TreeSet<Long>(Arrays.asList(
-        new Long[] { 1L })), descriptor.getProtocols().get("LinkAuth"));
+    assertEquals(new TreeSet<>(Arrays.asList(
+        1L, 2L, 3L, 4L)), descriptor.getProtocols().get("Link"));
+    assertEquals(new TreeSet<>(Arrays.asList(
+        1L)), descriptor.getProtocols().get("LinkAuth"));
     assertEquals(1325390599000L, descriptor.getPublishedMillis());
     assertEquals("D8733048FC8EC9102466AD8F3098622BF1BF71FD",
         descriptor.getFingerprint());
     assertEquals(48, descriptor.getUptime().longValue());
-    assertEquals(51200, (int) descriptor.getBandwidthRate());
-    assertEquals(51200, (int) descriptor.getBandwidthBurst());
-    assertEquals(53470, (int) descriptor.getBandwidthObserved());
+    assertEquals(51200, descriptor.getBandwidthRate());
+    assertEquals(51200, descriptor.getBandwidthBurst());
+    assertEquals(53470, descriptor.getBandwidthObserved());
     assertEquals("1469D1550738A25B1E7B47CDDBCD7B2899F51B74",
         descriptor.getExtraInfoDigestSha1Hex());
     assertTrue(descriptor.isHiddenServiceDir());
     assertEquals("Random Person <nobody AT example dot com>",
         descriptor.getContact());
-    assertEquals(Arrays.asList(new String[] {"reject *:*"}),
+    assertEquals(Arrays.asList("reject *:*"),
         descriptor.getExitPolicyLines());
     assertFalse(descriptor.isHibernating());
     assertNull(descriptor.getFamilyEntries());
@@ -516,9 +531,9 @@ public class ServerDescriptorImplTest {
         + "94.134.192.243 9001 0 0");
     assertEquals("saberrider2008", descriptor.getNickname());
     assertEquals("94.134.192.243", descriptor.getAddress());
-    assertEquals(9001, (int) descriptor.getOrPort());
-    assertEquals(0, (int) descriptor.getSocksPort());
-    assertEquals(0, (int) descriptor.getDirPort());
+    assertEquals(9001, descriptor.getOrPort());
+    assertEquals(0, descriptor.getSocksPort());
+    assertEquals(0, descriptor.getDirPort());
   }
 
   @Test
@@ -673,9 +688,9 @@ public class ServerDescriptorImplTest {
   public void testProtocolsOpt() throws DescriptorParseException {
     ServerDescriptor descriptor = DescriptorBuilder
         .createWithProtocolsLine("opt protocols Link 1 2 Circuit 1");
-    assertEquals(Arrays.asList(new Integer[] {1, 2}),
+    assertEquals(Arrays.asList(1, 2),
         descriptor.getLinkProtocolVersions());
-    assertEquals(Arrays.asList(new Integer[] {1}),
+    assertEquals(Arrays.asList(1),
         descriptor.getCircuitProtocolVersions());
   }
 
@@ -683,9 +698,9 @@ public class ServerDescriptorImplTest {
   public void testProtocolsNoOpt() throws DescriptorParseException {
     ServerDescriptor descriptor = DescriptorBuilder
         .createWithProtocolsLine("protocols Link 1 2 Circuit 1");
-    assertEquals(Arrays.asList(new Integer[] {1, 2}),
+    assertEquals(Arrays.asList(1, 2),
         descriptor.getLinkProtocolVersions());
-    assertEquals(Arrays.asList(new Integer[] {1}),
+    assertEquals(Arrays.asList(1),
         descriptor.getCircuitProtocolVersions());
   }
 
@@ -710,9 +725,9 @@ public class ServerDescriptorImplTest {
   public void testProtoGreenPurple() throws DescriptorParseException {
     ServerDescriptor descriptor = DescriptorBuilder
         .createWithProtoLine("proto Green=23 Purple=42");
-    assertEquals(new TreeSet<Long>(Arrays.asList(new Long[] { 23L })),
+    assertEquals(new TreeSet<>(Arrays.asList(23L)),
         descriptor.getProtocols().get("Green"));
-    assertEquals(new TreeSet<Long>(Arrays.asList(new Long[] { 42L })),
+    assertEquals(new TreeSet<>(Arrays.asList(42L)),
         descriptor.getProtocols().get("Purple"));
   }
 
@@ -876,9 +891,9 @@ public class ServerDescriptorImplTest {
   public void testBandwidthOpt() throws DescriptorParseException {
     ServerDescriptor descriptor = DescriptorBuilder
         .createWithBandwidthLine("opt bandwidth 51200 51200 53470");
-    assertEquals(51200, (int) descriptor.getBandwidthRate());
-    assertEquals(51200, (int) descriptor.getBandwidthBurst());
-    assertEquals(53470, (int) descriptor.getBandwidthObserved());
+    assertEquals(51200, descriptor.getBandwidthRate());
+    assertEquals(51200, descriptor.getBandwidthBurst());
+    assertEquals(53470, descriptor.getBandwidthObserved());
   }
 
   @Test
@@ -904,9 +919,9 @@ public class ServerDescriptorImplTest {
      * value. */
     ServerDescriptor descriptor = DescriptorBuilder
         .createWithBandwidthLine("bandwidth 51200 51200");
-    assertEquals(51200, (int) descriptor.getBandwidthRate());
-    assertEquals(51200, (int) descriptor.getBandwidthBurst());
-    assertEquals(-1, (int) descriptor.getBandwidthObserved());
+    assertEquals(51200, descriptor.getBandwidthRate());
+    assertEquals(51200, descriptor.getBandwidthBurst());
+    assertEquals(-1, descriptor.getBandwidthObserved());
   }
 
   @Test
@@ -1082,7 +1097,7 @@ public class ServerDescriptorImplTest {
       throws DescriptorParseException {
     ServerDescriptor descriptor = DescriptorBuilder
         .createWithExitPolicyLines("reject *:*\naccept *:*");
-    assertEquals(Arrays.asList(new String[] {"reject *:*", "accept *:*"}),
+    assertEquals(Arrays.asList("reject *:*", "accept *:*"),
         descriptor.getExitPolicyLines());
   }
 
@@ -1090,7 +1105,7 @@ public class ServerDescriptorImplTest {
   public void testExitPolicyOpt() throws DescriptorParseException {
     ServerDescriptor descriptor = DescriptorBuilder
         .createWithExitPolicyLines("opt reject *:*");
-    assertEquals(Arrays.asList(new String[] {"reject *:*"}),
+    assertEquals(Arrays.asList("reject *:*"),
         descriptor.getExitPolicyLines());
   }
 
@@ -1107,8 +1122,8 @@ public class ServerDescriptorImplTest {
       throws DescriptorParseException {
     ServerDescriptor descriptor = DescriptorBuilder
         .createWithExitPolicyLines("accept *:80\nreject *:*");
-    assertEquals(Arrays.asList(new String[] {"accept *:80",
-        "reject *:*"}), descriptor.getExitPolicyLines());
+    assertEquals(Arrays.asList("accept *:80",
+        "reject *:*"), descriptor.getExitPolicyLines());
   }
 
   @Test
@@ -1150,8 +1165,8 @@ public class ServerDescriptorImplTest {
     ServerDescriptor descriptor = DescriptorBuilder
         .createWithExitPolicyLines("reject 192.168.0.0/16:*\n"
         + "reject 94.134.192.243/255.255.255.0:*");
-    assertEquals(Arrays.asList(new String[] { "reject 192.168.0.0/16:*",
-        "reject 94.134.192.243/255.255.255.0:*"}),
+    assertEquals(Arrays.asList("reject 192.168.0.0/16:*",
+        "reject 94.134.192.243/255.255.255.0:*"),
         descriptor.getExitPolicyLines());
   }
 
@@ -1218,7 +1233,7 @@ public class ServerDescriptorImplTest {
   public void testFamilyOpt() throws DescriptorParseException {
     ServerDescriptor descriptor = DescriptorBuilder
         .createWithFamilyLine("opt family saberrider2008");
-    assertEquals(Arrays.asList(new String[] {"saberrider2008"}),
+    assertEquals(Arrays.asList("saberrider2008"),
         descriptor.getFamilyEntries());
   }
 
@@ -1227,8 +1242,7 @@ public class ServerDescriptorImplTest {
     ServerDescriptor descriptor = DescriptorBuilder
         .createWithFamilyLine("family "
         + "$D8733048FC8EC9102466AD8F3098622BF1BF71FD");
-    assertEquals(Arrays.asList(new String[] {
-        "$D8733048FC8EC9102466AD8F3098622BF1BF71FD"}),
+    assertEquals(Arrays.asList("$D8733048FC8EC9102466AD8F3098622BF1BF71FD"),
         descriptor.getFamilyEntries());
   }
 
@@ -1236,7 +1250,7 @@ public class ServerDescriptorImplTest {
   public void testFamilyNickname() throws DescriptorParseException {
     ServerDescriptor descriptor = DescriptorBuilder
         .createWithFamilyLine("family saberrider2008");
-    assertEquals(Arrays.asList(new String[] {"saberrider2008"}),
+    assertEquals(Arrays.asList("saberrider2008"),
         descriptor.getFamilyEntries());
   }
 
@@ -1273,8 +1287,8 @@ public class ServerDescriptorImplTest {
     ServerDescriptor descriptor = DescriptorBuilder
         .createWithFamilyLine("family "
         + "$D8733048FC8EC9102466AD8F3098622BF1BF71FD=saberrider2008");
-    assertEquals(Arrays.asList(new String[]
-        { "$D8733048FC8EC9102466AD8F3098622BF1BF71FD=saberrider2008" }),
+    assertEquals(Arrays.asList(
+        "$D8733048FC8EC9102466AD8F3098622BF1BF71FD=saberrider2008"),
         descriptor.getFamilyEntries());
   }
 
@@ -1284,8 +1298,8 @@ public class ServerDescriptorImplTest {
     ServerDescriptor descriptor = DescriptorBuilder
         .createWithFamilyLine("family "
         + "$D8733048FC8EC9102466AD8F3098622BF1BF71FD~saberrider2008");
-    assertEquals(Arrays.asList(new String[]
-        { "$D8733048FC8EC9102466AD8F3098622BF1BF71FD~saberrider2008" }),
+    assertEquals(Arrays.asList(
+        "$D8733048FC8EC9102466AD8F3098622BF1BF71FD~saberrider2008"),
         descriptor.getFamilyEntries());
   }
 
@@ -1298,9 +1312,9 @@ public class ServerDescriptorImplTest {
     assertNotNull(descriptor.getWriteHistory());
     BandwidthHistory parsedWriteHistory = descriptor.getWriteHistory();
     assertEquals(writeHistoryLine, parsedWriteHistory.getLine());
-    assertEquals(1325389904000L, (long) parsedWriteHistory
+    assertEquals(1325389904000L, parsedWriteHistory
         .getHistoryEndMillis());
-    assertEquals(900L, (long) parsedWriteHistory.getIntervalLength());
+    assertEquals(900L, parsedWriteHistory.getIntervalLength());
     SortedMap<Long, Long> bandwidthValues = parsedWriteHistory
         .getBandwidthValues();
     assertEquals(4345856L, (long) bandwidthValues.remove(1325387204000L));
@@ -1386,7 +1400,7 @@ public class ServerDescriptorImplTest {
     ServerDescriptor descriptor = DescriptorBuilder
         .createWithWriteHistoryLine("write-history 2012-01-01 03:51:44 "
         + "(900 s) ");
-    assertEquals(900, (long) descriptor.getWriteHistory()
+    assertEquals(900, descriptor.getWriteHistory()
         .getIntervalLength());
     assertTrue(descriptor.getWriteHistory().getBandwidthValues()
         .isEmpty());
@@ -1398,7 +1412,7 @@ public class ServerDescriptorImplTest {
     ServerDescriptor descriptor = DescriptorBuilder
         .createWithWriteHistoryLine("write-history 2012-01-01 03:51:44 "
         + "(900 s)");
-    assertEquals(900, (long) descriptor.getWriteHistory()
+    assertEquals(900, descriptor.getWriteHistory()
         .getIntervalLength());
     assertTrue(descriptor.getWriteHistory().getBandwidthValues()
         .isEmpty());
@@ -1426,7 +1440,7 @@ public class ServerDescriptorImplTest {
     ServerDescriptor descriptor = DescriptorBuilder
         .createWithWriteHistoryLine("write-history 2012-01-01 03:51:44 "
         + "(1800 s) 4345856");
-    assertEquals(1800L, (long) descriptor.getWriteHistory()
+    assertEquals(1800L, descriptor.getWriteHistory()
         .getIntervalLength());
   }
 
@@ -1439,9 +1453,9 @@ public class ServerDescriptorImplTest {
     assertNotNull(descriptor.getReadHistory());
     BandwidthHistory parsedReadHistory = descriptor.getReadHistory();
     assertEquals(readHistoryLine, parsedReadHistory.getLine());
-    assertEquals(1325389904000L, (long) parsedReadHistory
+    assertEquals(1325389904000L, parsedReadHistory
         .getHistoryEndMillis());
-    assertEquals(900L, (long) parsedReadHistory.getIntervalLength());
+    assertEquals(900L, parsedReadHistory.getIntervalLength());
     SortedMap<Long, Long> bandwidthValues = parsedReadHistory
         .getBandwidthValues();
     assertEquals(4268032L, (long) bandwidthValues.remove(1325387204000L));
@@ -1826,6 +1840,16 @@ public class ServerDescriptorImplTest {
         MASTER_KEY_ED25519_LINE, ROUTER_SIG_ED25519_LINE);
   }
 
+  @Test
+  public void testEd25519IdentityInvalidCrypto()
+      throws DescriptorParseException {
+    this.thrown.expect(DescriptorParseException.class);
+    this.thrown.expectMessage("CRYPTO_END before CRYPTO_BEGIN");
+    DescriptorBuilder.createWithEd25519Lines("identity-ed25519\n"
+        + "-----END ED25519 CERT-----\n-----BEGIN ED25519 CERT-----",
+        MASTER_KEY_ED25519_LINE, ROUTER_SIG_ED25519_LINE);
+  }
+
   @Test
   public void testEd25519MasterKeyMissing()
       throws DescriptorParseException {
@@ -1961,5 +1985,23 @@ public class ServerDescriptorImplTest {
     assertNull(descriptor.getDigestSha1Hex());
     assertNull(descriptor.getDigestSha256Base64());
   }
+
+  @Test
+  public void testBridgeDistributionRequestMoat()
+      throws DescriptorParseException {
+    ServerDescriptor descriptor =
+        DescriptorBuilder.createWithBridgeDistributionRequestLine(
+        "bridge-distribution-request moat");
+    assertEquals("moat", descriptor.getBridgeDistributionRequest());
+  }
+
+  @Test
+  public void testBridgeDistributionRequestEmptySpace()
+      throws DescriptorParseException {
+    this.thrown.expect(DescriptorParseException.class);
+    this.thrown.expectMessage("Illegal line 'bridge-distribution-request '.");
+    DescriptorBuilder.createWithBridgeDistributionRequestLine(
+        "bridge-distribution-request ");
+  }
 }
 
